//
// ScreensaverPlugin.cpp
// tmp
//
// Created by Bob Polis at 2020-10-26
// Copyright (c) 2020 SwiftCoder. All rights reserved.
//

#include "ScreensaverPlugin.hpp"
#include <libscnumerics.hpp>

void ScreensaverPlugin::setup(cairo_t* context, const cairo_rectangle_t& rect) {
    _c = context;
    _r = rect;
    make_black();
}

int ScreensaverPlugin::fps() const {
    return 30;
}

void ScreensaverPlugin::make_black() {
    cairo_set_source_rgb(_c, 0, 0, 0);
    cairo_rectangle(_c, 0, 0, _r.width, _r.height);
    cairo_fill(_c);
}

cairo_rectangle_t ScreensaverPlugin::random_rect() {
    return random_rect_in_rect(_r);
}

cairo_rectangle_t ScreensaverPlugin::random_rect_in_rect(const cairo_rectangle_t& rect) {
    cairo_rectangle_t r;
    std::uniform_real_distribution<double> dx {rect.x, rect.width};
    std::uniform_real_distribution<double> dy {rect.y, rect.height};
    double v1, v2;
    v1 = dx(sc::random::instance().engine());
    v2 = dx(sc::random::instance().engine());
    if (v1 < v2) {
        r.x = v1;
        r.width = v2 - v1;
    } else {
        r.x = v2;
        r.width = v1 - v2;
    }
    v1 = dy(sc::random::instance().engine());
    v2 = dy(sc::random::instance().engine());
    if (v1 < v2) {
        r.y = v1;
        r.height = v2 - v1;
    } else {
        r.y = v2;
        r.height = v1 - v2;
    }
    return r;
}

double ScreensaverPlugin::random_x() {
    std::uniform_real_distribution<double> dist {0, _r.width};
    return dist(sc::random::instance().engine());
}

double ScreensaverPlugin::random_y() {
    std::uniform_real_distribution<double> dist {0, _r.height};
    return dist(sc::random::instance().engine());
}

double ScreensaverPlugin::random01() {
    std::uniform_real_distribution<double> dist {};
    return dist(sc::random::instance().engine());
}

double ScreensaverPlugin::random_between(double v1, double v2) {
    std::uniform_real_distribution<double> dist {v1, v2};
    return dist(sc::random::instance().engine());
}

void ScreensaverPlugin::rounded_rect(const cairo_rectangle_t& rect, double radius) {
    double x0 = rect.x;
    double x1 = rect.x + radius;
    double x3 = rect.x + rect.width;
    double x2 = x3 - radius;
    double y0 = rect.y;
    double y3 = rect.y + rect.height;
    double y1 = rect.y + radius;
    double y2 = y3 - radius;
    cairo_move_to(_c, x1, y0);
    cairo_line_to(_c, x2, y0);
    cairo_arc(_c, x2, y1, radius, 1.5 * M_PI, 0);
    cairo_line_to(_c, x3, y2);
    cairo_arc(_c, x2, y2, radius, 0, 0.5 * M_PI);
    cairo_line_to(_c, x1, y3);
    cairo_arc(_c, x1, y2, radius, 0.5 * M_PI, M_PI);
    cairo_line_to(_c, x0, y1);
    cairo_arc(_c, x1, y1, radius, M_PI, 1.5 * M_PI);
}

void ScreensaverPlugin::config(const nlohmann::json& data) {
    _j = data;
    configure();
}

void ScreensaverPlugin::config(nlohmann::json&& data) {
    _j = data;
    configure();
}

void ScreensaverPlugin::configure() {
    // empty default implementation
}
